/*      PHOTOSHOP COMPOSITING RULE:
  
  Canvas Size   =   (numberOfFiles-1)*500  +  1000;
  move layer 000 all the way to the right, move highest number layer all the way to the left
  select all layers and align them to have equal spacing
  rever layer order
*/
import pitaru.sonia_v2_9.*;

boolean highQuality = true;

// geneic guitar thing demo.
// needs to map out points over time and save them out
// to be compiled in phtotoshop photomerge
int frameCounter, fileCounter, tickerCounter = 0;

PGraphics stage, bluredSquare;
float spacing, inc;
int noteWidth, noteHeight, lineInterval, lineCount, currColor, noteOpacity;
int boarder = 16;
BluredSquare note1, note2, note3, note4, note5, note6;
ColorManager Colors;
//type of viz:
String _TYPE = "dna"; //"dna", "rings", "arcs"
//font(s);
PFont font;
float setResolution = 72; //set resolution in ppi
float resolution = floater(setResolution)/72.0;
int stageHeight;
//guitar variables
int maxSamples = 1000;  //number of bins to sample from, starting at [0]
boolean[] peaks = new boolean[maxSamples]; //where to store peak data
float diamAmp  = 1.0; //effects radius;
float peakCrop = 0.045; //the sensitivity of where to crop peaks, 0.15 was "old" standard
int   binCrop  = 450;  //don't look at bin values over this number
float maxVal = 0.0;
int dapening = 10; //what spectrum value is divided by

//make saved Data array:
SavedNoteData[] SND = new SavedNoteData[0];
int currentFrame = 0;
int totalNotes = 0;

//intro stuff:
boolean running;
String user, email;
InputTextField nameTxt, emailTxt;

void setup() {
  size(round(1000*resolution),round((400*resolution)+boarder));
  running = false;
  font = loadFont("CourierNew36.vlw");
  stage = createGraphics(width,height,JAVA2D);
  println("*** NEW SESSION USING "+_TYPE+" VISUAL***");
  println("***STAGE SIZE "+width+" X "+height+"***");
  stage.smooth();
  smooth();
  stageHeight = height - boarder;
  noteOpacity = 75;
  Colors = new ColorManager();
  currColor = 5;
  Colors.setScheme(currColor);
  stage.background(Colors.getColor(0));
  spacing = 5.0;
  lineInterval = 50;
  lineCount = 0;
  inc = ((stageHeight)-(spacing*10.0))/9.0; 
  noteHeight = round(inc-spacing);
  noteWidth = round(20*resolution);
  Sonia.start(this);
  LiveInput.start(256*32);
  setupNotes();
  makeStartScreen();
}

void makeStartScreen() {
  textFont(font,25);
  fill(120);
  text("guitar mapper - capture engine",10,30);
  //make text fields
  nameTxt  = new InputTextField();
  emailTxt = new InputTextField();
  nameTxt.create(100,50,250,12);
  nameTxt.setLabel("name:");
  emailTxt.create(100,80,250,12);
  emailTxt.setLabel("email:");
}

void draw() {
  if (running == true) {
    //erase background:
    background(Colors.getColor(0)); 
    //get values for note(s):
    getSpectrum();
    //draw tickerlines, then notes:
    stage.beginDraw();
      drawGuides();
      //drawSpectrum(200,stage); //too ugly right now
      checkAllNotes();
    stage.endDraw();
    //copy pixels and shift to the right by 25%
    stage.copy(0,0,width,height,noteWidth/8,0,width,height);
    //fixes 'banding' bug for rings:
    stage.noStroke();
    stage.fill(0);
    stage.rect(0,0,noteWidth/8,stageHeight);
    image(stage,0,0);
    //save out images automatically
    if (frameCounter >= 100) {
      //saveImage();
    }else{
      frameCounter++;
    }
  } 
}




void drawNote(PGraphics target, BluredSquare note, int yVal) {
  int indent = round(50*resolution);
  if (highQuality == false) {
    target.image(note.attachImage(),indent,round(yVal*inc)); //quicker, uglier
  }else{
    target.blend(note.attachImage(),0,0,indent+noteWidth+20,(stageHeight),indent,round(yVal*inc),noteWidth+20,noteHeight+20,Colors.getBlend());  //slower, prettier
  }
}

void setupNotes() {
  note1 = new BluredSquare();
  note2 = new BluredSquare();
  note3 = new BluredSquare();
  note4 = new BluredSquare();
  note5 = new BluredSquare();
  note6 = new BluredSquare();
    int newW = round(10*resolution);
    int newH = round(35*resolution);
    int b1 = 2;
    int b2 = 3;
    note1.create(newW,newH,b1,b2,Colors.getColor(1),noteOpacity);
    note2.create(newW,newH,b1,b2,Colors.getColor(2),noteOpacity);
    note3.create(newW,newH,b1,b2,Colors.getColor(3),noteOpacity);
    note4.create(newW,newH,b1,b2,Colors.getColor(4),noteOpacity);
    note5.create(newW,newH,b1,b2,Colors.getColor(5),noteOpacity);
    note6.create(newW,newH,b1,b2,Colors.getColor(6),noteOpacity);
}

void drawGuides() {
  stage.stroke(Colors.getColor(7));
  stage.strokeWeight(1);
  stage.line(0,stageHeight,width,stageHeight);
  if (lineCount == 0) {
     stage.noStroke();
     stage.fill(Colors.getColor(7));
     //stage.ellipse(50,height-(boarder/2), boarder/3, boarder/3);
     stage.textFont(font,10);
     stage.text(tickerCounter, 50, height-5);
     tickerCounter++;
     lineCount = lineInterval;
   }else{
     lineCount--; 
   }
}

void saveImage(){
  String prefix;
  if (fileCounter<10) {
    prefix = "00";
  }else if (fileCounter < 100) {
   prefix = "0"; 
  }else{
   prefix =""; 
  }
  
  save("savedImages/section_"+prefix+fileCounter+".png");
  println("screenshot saved [section_"+prefix+fileCounter+".png]");
   fileCounter++;
   frameCounter = 0; 
}

float floater(float whole) {
 return(whole+0.0); 
}

void drawSpectrum(int range, PGraphics target) {
 PGraphics tempImg = createGraphics(40,stageHeight,JAVA2D);
 tempImg.beginDraw();
 tempImg.smooth();
 tempImg.stroke(Colors.getColor(7),10);
 tempImg.noFill();
 float vSpacing = ((stageHeight)/range);
 
 for (int i=0; i<range-1; i++) {
   float thisVal = LiveInput.spectrum[i]/10.0;
   if (thisVal > 20) {
    thisVal = 20.0; 
   }
   float nextVal = LiveInput.spectrum[i+1]/10.0;
   if (nextVal >20) {
    nextVal = 20.0; 
   }
   tempImg.line(20+thisVal,i*vSpacing,20-nextVal,(i+1)*vSpacing);
   //tempImg.ellipse(20,i*vSpacing,thisVal,thisVal);
 }
 tempImg.endDraw();
 target.blend(tempImg,0,0,width,height,30,0,40,stageHeight,OVERLAY);
}



////////////////////////// GUITAR & SONIA STUFF  /////////////////////////////////


////////////////   NOTE & STRING VALUES  ////////////////   
void checkAllNotes() {
  //updaye frame counter:
  currentFrame++;
  //create new SND object for this frameset:
  SavedNoteData newSND = new SavedNoteData();
  newSND.setFrame(currentFrame);
  SND = (SavedNoteData[]) expand(SND, SND.length+1);
  SND[SND.length-1] = newSND;
  
  int stringNum;
  // STRING #1 - E
  if(maxVal >0) {
  stringNum = 1;
  int[] array0 = {30, 61, 91};
  int[] array1 = {33, 66, 99, 132};
  int[] array2 = {35, 70, 105};
  int[] array3 = {37, 74, 111};
  int[] array4 = {39, 78, 117};
  int[] array5 = {41, 82, 123};
  int[] array6 = {44, 89, 134};
  int[] array7 = {47, 94, 141};
  int[] array8 = {49, 98};
  int[] array9 = {52, 104};
    checkNote(array0, 0, stringNum);
    checkNote(array1, 1, stringNum);
    checkNote(array2, 2, stringNum);
    checkNote(array3, 3, stringNum);
    checkNote(array4, 4, stringNum);
    checkNote(array5, 5, stringNum);
    checkNote(array6, 6, stringNum);
    checkNote(array7, 7, stringNum);
    checkNote(array8, 8, stringNum);
    checkNote(array9, 9, stringNum); 
  }
  
  // STRING #2 - B
  if(maxVal >0) {
  stringNum = 2;
  int[] array0 = {41, 81, 122};
  int[] array1 = {44, 89, 133, 222, 267};
  int[] array2 = {46, 93, 139, 185, 232, 278};
  int[] array3 = {49, 98, 148, 197, 246, 296};
  int[] array4 = {52, 104, 157, 209, 261};
  int[] array5 = {55, 110, 165, 221, 276};
  int[] array6 = {116, 174, 233, 291};
  int[] array7 = {62, 124, 248, 310};
  int[] array8 = {65, 131, 196, 262};
  int[] array9 = {69, 138, 207, 276};
    checkNote(array0, 0, stringNum);
    checkNote(array1, 1, stringNum);
    checkNote(array2, 2, stringNum);
    checkNote(array3, 3, stringNum);
    checkNote(array4, 4, stringNum);
    checkNote(array5, 5, stringNum);
    checkNote(array6, 6, stringNum);
    checkNote(array7, 7, stringNum);
    checkNote(array8, 8, stringNum);
    checkNote(array9, 9, stringNum); 
  } 
  
  // STRING #3 - D
  if(maxVal >0) {
  stringNum = 3;
  int[] array0 = {54, 109, 163};
  int[] array1 = {58, 117, 176, 234, 293, 352, 410};
  int[] array2 = {62, 124, 186, 310, 372};
  int[] array3 = {66, 132, 198, 329, 395};
  int[] array4 = {69, 139, 208, 348, 417};
  int[] array5 = {74, 147, 221, 295, 369};
  int[] array6 = {78, 156, 235, 313, 391};
  int[] array7 = {83, 166, 249, 415};
  int[] array8 = {88, 176, 352, 440};
  int[] array9 = {93, 186, 279, 373};
    checkNote(array0, 0, stringNum);
    checkNote(array1, 1, stringNum);
    checkNote(array2, 2, stringNum);
    checkNote(array3, 3, stringNum);
    checkNote(array4, 4, stringNum);
    checkNote(array5, 5, stringNum);
    checkNote(array6, 6, stringNum);
    checkNote(array7, 7, stringNum);
    checkNote(array8, 8, stringNum);
    checkNote(array9, 9, stringNum); 
  } 
  
 // STRING #4 - G
  if(maxVal >0) {
  stringNum = 4;
  int[] array0 = {72, 145, 218, 290};
  int[] array1 = {78, 156, 235, 313, 470};
  int[] array2 = {82, 165, 248, 496};
  int[] array3 = {88, 175, 263, 440, 528};
  int[] array4 = {93, 186, 372, 465};
  int[] array5 = {99, 198, 397, 497};
  int[] array6 = {105, 211, 422, 528};
  int[] array7 = {112, 224, 448};
  int[] array8 = {117, 235, 471};
  int[] array9 = {124, 248, 496};
    checkNote(array0, 0, stringNum);
    checkNote(array1, 1, stringNum);
    checkNote(array2, 2, stringNum);
    checkNote(array3, 3, stringNum);
    checkNote(array4, 4, stringNum);
    checkNote(array5, 5, stringNum);
    checkNote(array6, 6, stringNum);
    checkNote(array7, 7, stringNum);
    checkNote(array8, 8, stringNum);
    checkNote(array9, 9, stringNum); 
  }   
  
  // STRING #5 - A
  if(maxVal >0) {
  stringNum = 5;
  int[] array0 = {92, 183, 275, 366};
  int[] array1 = {99, 197, 296, 395, 494, 592, 691};
  int[] array2 = {104, 209, 313, 418, 522};
  int[] array3 = {110, 221, 331, 442, 663};
  int[] array4 = {116, 233, 349, 465, 582, 698};
  int[] array5 = {123, 247, 370, 493, 617};
  int[] array6 = {131, 261, 392, 654};
  int[] array7 = {139, 416, 555, 694};
  int[] array8 = {147, 294, 587, 734};
  int[] array9 = {156, 311, 623, 779};
    checkNote(array0, 0, stringNum);
    checkNote(array1, 1, stringNum);
    checkNote(array2, 2, stringNum);
    checkNote(array3, 3, stringNum);
    checkNote(array4, 4, stringNum);
    checkNote(array5, 5, stringNum);
    checkNote(array6, 6, stringNum);
    checkNote(array7, 7, stringNum);
    checkNote(array8, 8, stringNum);
    checkNote(array9, 9, stringNum); 
  } 
  
  // STRING #6 - HIGH E
  if(maxVal >0) {
  stringNum = 6;
  int[] array0 = {122, 245, 367};
  int[] array1 = {131, 262, 393, 523};
  int[] array2 = {138, 277, 415};
  int[] array3 = {146, 293, 440, 733};
  int[] array4 = {155, 311, 466, 777};
  int[] array5 = {164, 329, 493};
  int[] array6 = {175, 349, 524, 699, 874};
  int[] array7 = {184, 369, 554, 738};
  int[] array8 = {195, 391, 587, 782};
  int[] array9 = {207, 414, 829};
    checkNote(array0, 0, stringNum);
    checkNote(array1, 1, stringNum);
    checkNote(array2, 2, stringNum);
    checkNote(array3, 3, stringNum);
    checkNote(array4, 4, stringNum);
    checkNote(array5, 5, stringNum);
    checkNote(array6, 6, stringNum);
    checkNote(array7, 7, stringNum);
    checkNote(array8, 8, stringNum);
    checkNote(array9, 9, stringNum);
  }
  /*
  for (int i=0; i<SND[currentFrame-1].getTotalNotes(); i++) {
    print(SND[currentFrame-1].getString(i)+", "); 
  }
  println("");
 */
}

void checkNote(int[] peakVals, int c,int s){
  //samples determins how many peaks are used
  int samples = peakVals.length;
  //diam is all the peak values divided by samples
  float diam = 0.0;
  //if any val is false, notePlayed changes to false;
  boolean notePlayed = true;
  //int[] array = {val1,val2,val3,val4,val5};
  for (int i=0; i<samples; i++) {
     if (peaks[peakVals[i]] == false && peakVals[i] < binCrop) {
       notePlayed = false; 
     }
     diam+=(LiveInput.spectrum[peakVals[i]]);
  } 
  diam = diam/floater(samples);
  //check to see if note has been played:
  if (notePlayed == true) {
   //save data to array:
   totalNotes++;
   SND[currentFrame-1].pushNote(s,c,diam);
   //draw it out:
   if (_TYPE == "dna") {
     makeNote(s,c,diam);
   }else if(_TYPE == "rings" || _TYPE == "arcs") {
    makeCircleNote(s,c,diam); 
   }
  }
}


void makeCircleNote(int string, int fret, float val) {
  float indent = 100.0;
  float diam = val/20.0;
  if (_TYPE == "rings") {
    diam = val/40.0;
    if (diam >= indent) {
      diam = indent;
    }
  }
  stage.noFill();
  stage.stroke(Colors.getColor(string),noteOpacity);
  stage.ellipse(50,(fret*inc)+(inc*0.5),diam,diam);
}

void makeNote(int string, int fret, float val) {
  if (string == 1) {  
    drawNote(stage, note1, fret);
  }else if (string == 2) {  
    drawNote(stage, note2, fret);
  }else if (string == 3) {  
    drawNote(stage, note3, fret);
  }else if (string == 4) {  
    drawNote(stage, note4, fret);
  }else if (string == 5) {  
    drawNote(stage, note5, fret);
  }if (string == 6) {  
    drawNote(stage, note6, fret);
  }
}
  
void getSpectrum() {
   LiveInput.getSpectrum();
   // draw a bar for each of the elements in the spectrum array.
   // Note - the current FFT math is done in Java and is very raw. expect optimized alternative soon.
   // get max value to use as comparison.
   for ( int i = 0; i < maxSamples; i++ ){
    if (LiveInput.spectrum[i]/dapening > maxVal) {
     maxVal = (LiveInput.spectrum[i])/dapening; 
    }
   }
   //make empty array:
   //boolean[] peaks = new boolean[200];
   for ( int i = 0; i < maxSamples; i++){
    //get relative value, and color code & assign in array.
     float relVal =  ((LiveInput.spectrum[i])/dapening)/maxVal;
     if (relVal >= peakCrop) {
        peaks[i] = true;
      }else{
        peaks[i] = false;
      }
   }
}

public void stop(){
    Sonia.stop();
    super.stop();
}
    
void keyPressed() {
  if (running == true) {
  //change note detectin accuracy
  if (keyCode == UP && peakCrop > 0) {
    peakCrop -= 0.005;
    println("peakCrop decreased to: "+peakCrop);
  }
  if (keyCode == DOWN) {
     peakCrop += 0.005;
     println("peakCrop increased to: "+peakCrop); 
  }
  //cycle colors
  if (key == 'c') {
    if (currColor == Colors.getTotalColors()) {
     currColor = 1; 
    }else{
      currColor++;
    }
    println("color scheme set to #"+currColor+" ("+Colors.getDiscription(currColor)+")");
    Colors.setScheme(currColor);
    setupNotes();
  }
  if (keyCode == RIGHT) {
   noteOpacity += 5;
   if (noteOpacity > 255) {
    noteOpacity = 0; 
   }
   println("note opacity changed to: "+noteOpacity);
   setupNotes(); 
  }
  
  if (keyCode == LEFT) {
   noteOpacity -= 5;
   if (noteOpacity < 0) {
    noteOpacity = 255; 
   }
   println("note opacity changed to: "+noteOpacity);
   setupNotes(); 
  }
  
  if (key == 'q') {
    exportData(); 
  }
  
  //END RUNNING COMMANDS
  }else{
    
  // FROM INTRO SCREEN 
  if (key != CODED && key != 10) {
    nameTxt.recieveInput(key);
    emailTxt.recieveInput(key);
  }else{
     if (keyCode == DELETE || keyCode == BACKSPACE || keyCode == LEFT){
       nameTxt.backspace();
       emailTxt.backspace();
     }else if (keyCode == RETURN || keyCode == ENTER || key==10) {
       //10 is also return on mac?
         user = nameTxt.getValue();
         email = emailTxt.getValue();
         running = true;  
     }
  }

    
    
  }
}


void exportData() {
 println("writting XML..."); 
 //auto incriment files:
 String files[] = loadStrings("../saved_sessions/_meta.txt");
 int newNumber  = files.length+1;
 String newName;
   if (newNumber < 10) {
      newName = "session000"+newNumber; 
   }else if (newNumber < 100) {
      newName = "session00"+newNumber; 
   }else if (newNumber < 1000) {
      newName = "session0"+newNumber; 
   }else{
      newName = "session"+newNumber; 
   };
 files = expand(files, files.length+1);
 files[files.length-1] = newName;
 saveStrings("../saved_sessions/_meta.txt",files);
 //make file:
 PrintWriter output = createWriter("../saved_sessions/"+newName+"/data.xml");
 output.println("<?xml version='1.0'?>");
 output.println("<session>");
 //create session data:
 output.println("  <sessionData>");
 output.println("    <meta date="+month()+"-"+day()+"-"+year()+" time="+hour()+":"+minute()+":"+second()+" user="+user+" userEmail="+email+" totalFrames="+currentFrame+" totalNotes="+totalNotes+" />");
 output.println("  </sessionData>");
 //create saved data:
 output.println("  <savedNoteData>");
 for (int i=0; i < SND.length; i++) {
  output.println("    <frame id="+SND[i].getFrame()+">");
  for (int n=0; n < SND[i].getTotalNotes(); n++) {
    output.println("      <note string="+SND[i].getString(n)+" fret="+SND[i].getFret(n)+" value="+SND[i].getValue(n)+" />");
  }
  output.println("    </frame>"); 
 }
 output.println("  </savedNoteData>");
 output.println("</session>");
 //finished, create file:
 println("finishing and closing");
 output.flush();
 output.close();
 exit();
}

void mousePressed() {
  if (running != true) {
      nameTxt.setFocus();
      emailTxt.setFocus();
  }
}
