/*      PHOTOSHOP COMPOSITING RULE:
  
  Canvas Size   =   (numberOfFiles-1)*500  +  width;
  move layer 000 all the way to the right, move highest number layer all the way to the left
  select all layers and align them to have equal spacing
  rever layer order
*/

import processing.xml.*;
XMLElement xml;

//SET BUY USER//
String folderName = "session0001";
float setResolution = 72.0; //set resolution in ppi
float inchesTall = 6.0;
int noteOpacity = 100;
////////////////
int boarder = 16;
float resolution = setResolution/72.0;
//////////////////
int currentFrame, lastFrame, stageHeight, stageWidth, noteWidth, noteHeight, lineInterval, lineCount, frameCounter, fileCounter, tickerCounter;
float inc, spacing;
PGraphics stage;
PFont font;
boolean running;
//custome objects
BluredSquare note1, note2, note3, note4, note5, note6;
ColorManager Colors;
InputTextField sessionTxt, colorTxt, heightTxt, resTxt;

float userSetHeight;
int userSetRes;

//renderer
void setup() {
 size(round(500*resolution),round((inchesTall*setResolution)+boarder));
 smooth();
 stageHeight = height - boarder;
 stageWidth  = round(500*resolution);
 font = loadFont("CourierNew36.vlw");
 currentFrame = 1;
 frameCounter = 0;
 Colors = new ColorManager();
 Colors.setScheme(5);
 background(150);
 spacing = 5.0;
 lineInterval = 50;
 lineCount = 0;
 fileCounter = 0;
 tickerCounter = 0;
 startScreen();
 running = false;
}

void loadXMLandStart() {
   //stage = createGraphics(width,height,JAVA2D);
   stageHeight = round(userSetHeight*userSetRes);
   stageWidth  = round(500*((userSetRes+0.0)/72.0));
   resolution = (userSetRes+0.0)/72;
   stage = createGraphics(stageWidth,(stageHeight+boarder),JAVA2D);
   stage.smooth();
   String xmlPath = "../saved_sessions/"+folderName+"/data.xml";
   xml = new XMLElement(this, xmlPath);
   lastFrame = xml.getChild(1).getChildCount();
   stage.background(Colors.getColor(0));
   background(Colors.getColor(0));
   
   inc = ((stageHeight)-((spacing*resolution)*10.0))/9.0; 
   noteHeight = round((inc-spacing)*1.75);
   noteWidth = round(noteHeight/4.0);
   setupNotes();
   
   running = true;
   
   println("***STARTING RENDER***");
   println(userSetHeight+" inches tall @ "+userSetRes+" ppi");
   println("using color pallet #"+ colorTxt.getValue() +" '"+Colors.getDiscription(int(colorTxt.getValue()))+"'");
   println("captured on "+xml.getChild(0).getChild(0).getAttribute("date")+" at "+xml.getChild(0).getChild(0).getAttribute("time"));
   println("recorded by "+xml.getChild(0).getChild(0).getAttribute("user")+" ("+xml.getChild(0).getChild(0).getAttribute("userEmail")+")");
}

void startScreen() {
  sessionTxt = new InputTextField();
  colorTxt = new InputTextField();
  heightTxt = new InputTextField();
  resTxt = new InputTextField();
  sessionTxt.create(100,50,200,12);
  sessionTxt.setLabel("session folder:");
  sessionTxt.setText("session0001");
  colorTxt.create(100,80,50,12);
  colorTxt.setLabel("color scheme # (1 - "+Colors.getTotalColors()+"):");
  colorTxt.setText("6");
  
  heightTxt.create(100,110,50,12);
  heightTxt.setLabel("height (inches):");
  heightTxt.setText("4.0");
  resTxt.create(100,140,50,12);
  resTxt.setLabel("resolution (ppi):");
  resTxt.setText("72");
}

void draw() {
  if (running == true) {
  if (currentFrame < lastFrame) {
    render();
  }else if (currentFrame == lastFrame){
    saveImage();
    println("***FINISHED RENDERING***");
  }
  currentFrame++;
  }
}

void render() {
 //make the image
 //erase background:
  background(Colors.getColor(0));  
  stage.beginDraw(); 
   drawGuides(); 
   for (int i=0; i<xml.getChild(1).getChild(currentFrame).getChildCount(); i++) {
     XMLElement note=xml.getChild(1).getChild(currentFrame).getChild(i);
     int s = note.getIntAttribute("string");
     int f = note.getIntAttribute("fret");
     float v = note.getFloatAttribute("value");
     makeNote(s,f,v);
   }
  stage.endDraw();
  stage.copy(0,0,stageWidth,stageHeight+boarder,noteWidth/8,0,stageWidth,stageHeight+boarder);
  //fixes 'banding' bug for rings:
  stage.noStroke();
  stage.fill(Colors.getColor(0));
  stage.rect(0,0,10/8,stageHeight);
  image(stage,0,0);
  //save out images automatically
  if (frameCounter >= 100) {
    saveImage();
  }else{
    frameCounter++;
  } 
}


//////////
void drawNote(PGraphics target, BluredSquare note, int yVal) {
  int indent = round(50*resolution);
  target.blend(note.attachImage(),0,0,indent+noteWidth+20,(stageHeight),indent,round(yVal*inc),noteWidth+20,noteHeight+20,Colors.getBlend());  //slower, prettier
}

void makeNote(int string, int fret, float val) {
  if (string == 1) {  
    drawNote(stage, note1, fret);
  }else if (string == 2) {  
    drawNote(stage, note2, fret);
  }else if (string == 3) {  
    drawNote(stage, note3, fret);
  }else if (string == 4) {  
    drawNote(stage, note4, fret);
  }else if (string == 5) {  
    drawNote(stage, note5, fret);
  }if (string == 6) {  
    drawNote(stage, note6, fret);
  }
}

void setupNotes() {
  note1 = new BluredSquare();
  note2 = new BluredSquare();
  note3 = new BluredSquare();
  note4 = new BluredSquare();
  note5 = new BluredSquare();
  note6 = new BluredSquare();
    int newW = noteWidth; //round(10*resolution);
    int newH = noteHeight; //round(35*resolution);
    int b1   = round(2*resolution);
    int b2   = 3;
    note1.create(newW,newH,b1,b2,Colors.getColor(1),noteOpacity);
    note2.create(newW,newH,b1,b2,Colors.getColor(2),noteOpacity);
    note3.create(newW,newH,b1,b2,Colors.getColor(3),noteOpacity);
    note4.create(newW,newH,b1,b2,Colors.getColor(4),noteOpacity);
    note5.create(newW,newH,b1,b2,Colors.getColor(5),noteOpacity);
    note6.create(newW,newH,b1,b2,Colors.getColor(6),noteOpacity);
}

void drawGuides() {
  stage.stroke(Colors.getColor(7));
  stage.strokeWeight(1);
  stage.line(0,stageHeight,stageWidth,stageHeight);
  if (lineCount == 0) {
     stage.noStroke();
     stage.fill(Colors.getColor(7));
     //stage.ellipse(50,height-(boarder/2), boarder/3, boarder/3);
     stage.textFont(font,10);
     stage.text(tickerCounter, 50, (stageHeight+boarder)-5);
     tickerCounter++;
     lineCount = lineInterval;
   }else{
     lineCount--; 
   }
}


void saveImage(){
  String prefix;
  if (fileCounter<10) {
    prefix = "00";
  }else if (fileCounter < 100) {
   prefix = "0"; 
  }else{
   prefix =""; 
  }
  
  stage.save("../saved_sessions/"+folderName+"/renders/"+prefix+fileCounter+".png");
  println("  screenshot saved ["+folderName+"/renders/"+prefix+fileCounter+".png]");
   fileCounter++;
   frameCounter = 0; 
}


void keyPressed() {
 if (running != true) {
  if (key != CODED && key != 10) {
    sessionTxt.recieveInput(key);
    colorTxt.recieveInput(key);
    heightTxt.recieveInput(key);
    resTxt.recieveInput(key);
  }else{
     if (keyCode == DELETE || keyCode == BACKSPACE || keyCode == LEFT){
       sessionTxt.backspace();
       colorTxt.backspace();
       heightTxt.backspace();
       resTxt.backspace();
     }else if (keyCode == RETURN || keyCode == ENTER || key==10) {
       //10 is also return on mac?
         folderName = sessionTxt.getValue();
         Colors.setScheme(int(colorTxt.getValue()));
         userSetHeight = float(heightTxt.getValue());
         userSetRes = int(resTxt.getValue());
         loadXMLandStart();
     }
  }
 } 
}

void mousePressed() {
   if (running != true) {
      sessionTxt.setFocus();
      colorTxt.setFocus();
      heightTxt.setFocus();
      resTxt.setFocus();
  }
}
